/* LPCC pro DTW rozpoznavac. Nacte jeden file, udela z nej LPCC
   fea_lpcc x.l16(nebo x.wav) x.cps

   CPS file ma na zacatku 1 int s poctem ramcu, pak vektory. 
*/ 

#include <stdio.h>
#include <strings.h>
#include "wavframes.h"   /* waveform input and framing */
#include "speech.h"      /* speech processing */
#include "config.h"      /* all config parameters */ 

/* ------------------------------------------------------------------ */
int main (int argc, char **argv) {
  char *namein;
  char *nameout; 
  FILE *fileout; 
  int res; 
  float *frame;   /* speech frame - already floats */ 
  float *params;  /* out parameters - here just a copy of samples */
  int frmcnt=0, i; 
  char *extension; 
  float BigMatrix[MAXFRAMES][LPCC_COEFFS]; 

  if (argc != 3) {
    fprintf (stderr, "Usage: fea_lpcc infile(raw or WAV) outfile.cps\n");
    exit (1); 
  }

  namein = argv[1]; 
  nameout = argv[2]; 

  /* the opening function is different for RAW and WAV ... */
  extension = namein + strlen(namein) - 3; /* the extension has 3 chars */
  if (!strcasecmp (extension, "wav")) {
    fprintf (stderr, "WAV FILE: "); 
    open_wav_file (namein); 
  }
  else {
    fprintf (stderr, "RAW FILE: "); 
    open_raw_file (namein);
  } 
  frame = allocate_speech_frame (); 

  /* process frames */ 
  frmcnt = 0; 
  while (new_frame(frame)) {
    process_frame (frame, BigMatrix[frmcnt++]); 
    if (frmcnt >= MAXFRAMES) {
      printf ("Sorry, no of frames bigger than %d, exiting ...\n", MAXFRAMES);
    }
  }

  close_wav_file(); 
  free(frame);

  /* write everything out */
  fileout = fopen(nameout, "w");
  if (fileout == NULL) {
    fprintf (stderr, "problem opening output CPS file `%s'\n", nameout); 
    exit (1); 
  }
  /* write the no of frames */
  fwrite (&frmcnt, sizeof(int), 1, fileout);
  /* write the vectors */
  for (i=0; i<frmcnt; i++) 
    fwrite (BigMatrix[i], sizeof(float), LPCC_COEFFS, fileout);
  fclose (fileout); 

  fprintf (stderr, "%s -> %s %d frames\n",namein,nameout,frmcnt); 
  return (0); 
}
