#include "lpcc.h"
#include "common.h"

   /********************************************************************
   * FUNCTION: LpcToCps
   * PURPOSE: lpc -> lpcc coefficients
   * INPUTS:  
   *    pfA[0:swLpcOrder-1] - lpc coeffs. a1...aP
   *    swLpcOrder - order of Lpc analysis
   *    swCpsNumber - number of lpcc coeffs to compute - usually  the same as 
   *                  swLpcOrder
   *
   * OUTPUT: 
   *    pfC[0:swCpsNumber-1] - lpcc coeffs. c1...cQ
   *
   * RETURN: 
   *    none
   *
   * DESCRIPTION:
   * KEYWORDS:
   *********************************************************************/

void LpcToCps (float *pfA, float *pfC, short swLpcOrder, short swCpsNumber)
{
   short i,j,iindex,jindex;
   float sum;
   static float *help;
   static short swFirstPass=1;
   
   if (swFirstPass) {
      help=Alloc_1_Float (swCpsNumber);
      swFirstPass = 0;
   }
   
   for (i=0; i < (swCpsNumber < swLpcOrder ? swCpsNumber : swLpcOrder); i++)
      help[i]=pfA[i];
   for (j=i; j< swCpsNumber; j++)   /* fill the rest (if any) with zeros */
      help[j]=0.0;

   for (i=1, iindex=0; i <= swCpsNumber; i++, iindex++) { /* i,j - theoretical
                                                             indices, xindex -
                                                             addressing */
      sum=0.0;
      for(j=1, jindex=0; j <= (i-1); j++, jindex++)
         sum += ( (float)j / (float)i ) * help[i-j-1]*pfC[jindex];
      pfC[iindex] = - help[iindex] - sum;
   }
}
