/* DTW rozpoznavac. Nacte urcity pocet referenci, vse ostatni poklada za test
   soubory, vsechny testy rozpozna. 
   volani: 

   reco_dtw N ref1.cps ref2.cps ... refN.cps test1.cps test2.cps ... 
*/ 

#include <stdio.h>
#include "config.h"      /* all config parameters */ 
#include "dtw.h"      /* all config parameters */ 
#include "common.h"      /* all config parameters */ 

/* allocates and reads the lpcc matrix, N will contain the no of vectors. */
float ** ReadLPCCMatrix (char *namein, int *N) {
  float **mat; 
  FILE *filein; 
  int i; 

  filein = fopen(namein, "r");
  if (filein == NULL) {
    fprintf (stderr, "problem opening input CPS file `%s'\n", namein); 
    exit (1); 
  }
  /* read no of frames */
  fread(N, sizeof(int), 1, filein); 

  /* allocate and read vectors */
  mat = Alloc_2_Float(*N, LPCC_COEFFS); 
  for (i=0; i<*N; i++) 
    fread(mat[i], sizeof(float), LPCC_COEFFS, filein); 
  fclose(filein); 
  return mat; 
}

/* ------------------------------------------------------------------ */
int main (int argc, char **argv) {
  int Nrefs, Ntests;     /* number of references and tests */
  int argptr; 
  float ***refs;  /* references - array of matrices */
  int *lrefs;     /*   lengths of references */
  char **rnames;  /* names of references */
  float **test;   /* test - just one matrix */
  int ltest;      /*   length of test */
  char *tname; 
  int i,j; 
  float dtwdist; 
  float bestdtw;  /* best DTW distance */
  int ibestdtw;   /* and its index */

  /* get the numbers */
  Nrefs = atoi(argv[1]); 
  Ntests = argc - Nrefs - 2;
  argptr = 2;               /* points to the first ref */

  /* allocate */
  refs = (float***) malloc (Nrefs * sizeof(float**));  /* can't use Alloc_3_Float */
  lrefs = Alloc_1_Int (Nrefs); 
  rnames = (char**) malloc (Nrefs * sizeof(char*));

  /* read and process references */
  printf ("LOADING REFERENCES\n"); 
  for (i=0; i<Nrefs; i++) {
    rnames[i] = argv[argptr++]; 
    printf ("reading reference %d: %s ...",i,rnames[i]); 
    refs[i] = ReadLPCCMatrix (rnames[i], &lrefs[i]); 
    printf (" done: %d frames\n", lrefs[i]); 
  }

  /* now the recognition ! */
  printf ("\nRECOGNITION\n"); 

  for (i=0; i<Ntests; i++) {
    /* reading */
    tname = argv[argptr++]; 
    printf ("reading test %d: %s ...",i,tname); 
    test = ReadLPCCMatrix (tname, &ltest); 
    printf (" done: %d frames\n", ltest); 

    bestdtw = BIG_FLOAT_VALUE; 
    ibestdtw = -1; 

    /* running DTW for all references */
    for (j=0; j<Nrefs; j++) {
      dtwdist = DTWDistance (refs[j], test, lrefs[j], ltest, LPCC_COEFFS, NULL);

      /* here is "THE RECOGNITION" */
      if (dtwdist < bestdtw) {
	bestdtw = dtwdist;
	ibestdtw = j; 
      }
      /* print it out */
      printf ("\tDTW(%s,%s)=%f\n", rnames[j], tname, dtwdist); 
    }
    /* end for all refs - we have the winner */
    printf ("*** RECOGNIZED: %s, best distance %f\n\n", rnames[ibestdtw], bestdtw); 

  } /* all for all tests */
  return (0); 
}
